import sys
import logging
import logging.handlers
import platform
from healthcheck.healthservice import HealthService
from optparse import OptionParser

logger = logging.getLogger('healthcheck')


def run(args):
    long_description = """healthcheck is your friendly neighborhood daemon that has a dirty job to do.
healthcheck runs user-defined scripts at a specified interval. If any of the scripts fails,
healthcheck will shutdown the machine. You can use the `jetpack keepalive` command to delay or
cancel system shutdown.
"""
    parser = OptionParser()
    parser.set_description(long_description)
    parser.add_option('-p', '--pidfile', help='file to store pid in',
                      dest='pidfile')
    parser.add_option('--logfile', dest='logfile', help='file to direct logging output to')
    parser.add_option('-d', '--daemonize', dest="daemonize", default=False,
                      action='store_true', help='daemonize healthcheck')
    parser.add_option('-t', '--timeout', dest='timeout', default='2',
                      help='Timeout in minutes for each script')
    parser.add_option('-i', '--interval', dest='interval', default='10',
                      help='execution interval')
    parser.add_option('-s', '--scriptdir', dest='scriptdir',
                      help='Directory holding scripts to execute')
    parser.add_option('--stay-of-execution', dest='stay_of_execution',
                      help='Number of minutes to wait before terminating the machine',
                      default=1)
    parser.add_option('--dry-run', dest='dry_run', default=False,
                      action='store_true', help='Run scripts but do not shutdown the system on failure')
    parser.add_option('--verbose', action='store_true', dest='debug', default=False)
    (options, args) = parser.parse_args()

    if options.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    health_service = HealthService(options)

    if options.logfile:
        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
        handler = logging.handlers.RotatingFileHandler(options.logfile, maxBytes=1024 * 1024 * 50, backupCount=5)
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    if options.daemonize is True:
        health_service.run()
    else:
        health_service.execute()

if __name__ == "__main__":
    run(sys.argv[0:])
