import logging
import threading
import platform
import subprocess
import _thread
import os

logger = logging.getLogger('healthcheck')


class CheckError(Exception):

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class Check:

    def popen(self, command):
        logger.debug('popen called with %s' % command)
        script_name, script_ext = os.path.splitext(os.path.basename(command))
        logger.debug('script_name is: %s' % script_name)
        scratch = os.path.join(self.scratch, script_name)
        try:
            os.makedirs(scratch, 0o755)
            os.chmod(scratch, 0o755)
        except OSError as e:
            if e.errno == 17:
                pass
            else:
                raise(e)

        if platform.os.name == 'nt':
            if script_ext == '.py':
                if 'CYCLECLOUD_HOME' in os.environ:
                    jetpack_home = os.environ['CYCLECLOUD_HOME']
                else:
                    # Hope it's somewhere
                    jetpack_home = 'C:\\cycle\\jetpack'
                command = os.path.join(jetpack_home, 'system', 'embedded', 'python', 'python.exe') \
                        + ' ' + command
                logger.debug('Using %s to launch python script' % command) 
            return subprocess.Popen(command, shell=True, cwd=self.scratch, text=True)
        else:
            os.chmod(command, 0o755)
            return subprocess.Popen(command, cwd=self.scratch, stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE, text=True)

    def run(self):
        def target():
            logger.info('Executing script {0}'.format(self.script))
            self.process = self.popen(self.script)
            self.stdout, self.stderr = self.process.communicate()
            self.returncode = self.process.returncode

        try:
            t = threading.Thread(target=target)
            t.start()
            t.join(int(self.timeout))
        except RuntimeError:
            raise CheckError("Can't execute healtcheck script as cannot create new thread")

        if t.is_alive():
            log_message = 'Script {0} timed out after {1} minutes, terminating process'
            logger.critical(log_message.format(self.script, self.timeout))
            self.process.terminate()
            t.join()
            return 0
        return self.returncode

    def __init__(self, script, timeout, scratch):
        self.script = script
        self.timeout = int(timeout) * 60
        self.process = None
        self.scratch = scratch
