# Copyright 2014, Cycle Computing
# All rights reserved - Do Not Redistribute
#
import json
import os
import platform
import logging
import shutil
import jetpack.util
import jetpack.converge.userdata
import jetpack.config


logger = logging.getLogger()


class JetpackConfigureError(Exception):
    pass


class JetpackConfigure(object):

    def __init__(self):
        self.cyclecloud_home = ""
        self.cyclecloud_bootstrap = ""

    def _set_paths(self):
        # Determine the home and bootstrap directories
        self.cyclecloud_home = 'C:\\cycle\\jetpack' if jetpack.util.is_windows() else '/opt/cycle/jetpack'
        self.cyclecloud_bootstrap = os.path.join(self.cyclecloud_home, "system", "bootstrap")

        self.cyclecloud_home = os.environ.get('CYCLECLOUD_HOME', self.cyclecloud_home)
        self.cyclecloud_bootstrap = os.environ.get('CYCLECLOUD_BOOTSTRAP', self.cyclecloud_bootstrap)

    def _transform_userdata_to_dict(self):
        user_data = jetpack.converge.userdata.cache_userdata().get('user_data')

        identity_settings = {}
        identity_settings["cluster_name"] = user_data["pool"]["name"]
        identity_settings["instance_id"] = jetpack.util.get_instance_id()

        cs_settings = {}
        cs_settings['webserver'] = user_data['config']['webServer']  # TODO: This should be moved into settings or config
        cs_settings['api_username'] = jetpack.config.get('cyclecloud.config.username')
        cs_settings['api_password'] = jetpack.config.get('cyclecloud.config.password')
    
        identity_settings['template'] = jetpack.config.get('cyclecloud.node.template')
        identity_settings['cluster_start_count'] = jetpack.config.get('cyclecloud.cluster.start_count')
        identity_settings['cluster_session_id'] = jetpack.config.get('cyclecloud.cluster.session_id')

        config_dict = {}
        config_dict["identity"] = identity_settings
        config_dict["cycle_server"] = cs_settings

        return config_dict

    def run(self, user_data_path=None, jetpack_config=None):
        self._set_paths()

        config_dict = self._transform_userdata_to_dict()
        jetpack.util.write_config(config_dict, jetpack_config)


def execute(user_data_path=None, jetpack_config=None):
    config = jetpack.util.parse_config(jetpack_config)

    if not config:
        jetpack_configure = JetpackConfigure()
        jetpack_configure.run(user_data_path=user_data_path, jetpack_config=jetpack_config)

        # Clean up cyclecloud_data directory since we've finished everything for bootstrapping
        if os.path.isdir(jetpack.util.get_installer_dir()):
            shutil.rmtree(jetpack.util.get_installer_dir())

    return jetpack.util.parse_config(jetpack_config)
