import os
from jetpack import messaging
import json
import logging

logger = logging.getLogger()


class SendError(Exception):
    pass


def send(message=None, file=None, routing_key=None, exchange=None, config=None):
    """ Deprecated: Sends a message over an amqp bus

    If the routing key starts with "cyclecloud.service.status" a internal message is sent, otherwise 
    an exception is raised that this method has been deprecated """

    if not message and not file:
        raise SendError("No message body specified")
    elif message and file:
        raise SendError("Cannot specify both a message and a file at the same time")

    if file:
        filename = os.path.expanduser(file)
        with open(filename, 'r') as content_file:
            message = content_file.read()

    # kludge: if this is being used to send status messages, we just convert them to a direct POST as a system message.
    # This is so later versions of CycleCloud can drop the AMQP broker since it is not used 
    if routing_key and routing_key.startswith("cyclecloud.service.status"):
        data = json.loads(message)
        if "cluster_name" in data:
            del data["cluster_name"]
        messaging.send_internal_message(data, "service_status")
    else:
        raise NotImplementedError("Sending direct AMQP messages has been removed, please use a system message instead.")